function [Imc, pd]= simstate_ms_lik(lik)

% simulate multivariate (M) indicators with L states 
% from the posterior distribution proportional to 
% likelihood*unrestricted Markov switching prior

% (all indicators are assumed to start in state j with probability 1/L)

% the procedure automatically recognizes from the third dimesnion of the transition matrix xi
% whether the transition matrices of the different processes are different or not

% Sampling method used:
% 1. multi move sampling (forward filtering - backward sampling)
% 2. if xi is the same for all processes the procedure is vectorized and 
%    samples all components I(t,:) simultaniously, 
%    otherwise there is an additional loop over m=1,...,M

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input: 

%    row k of xi(:,:,m) corresponds to the conditional probability of I(t,m) given I(t-1,m)=k

% lik .. likelihood (L times M times T)

%  lik(k,m,t) is the log-likelihood of all data at time t which are relevant for process m,
%    if we assume that the process m takes the value k at time t

% Output:

% Imc ... path of the indicator sampled from the likelihood (T times M);
% comments:
% 1.  Imc(t,m) refers to the value of process m at time t

% sp ... sum of the logarithm of all normalizing constants obtained during filtering (1 times M)
% comments:
% 1. sp(1,m) is identical with the log of the marginal likelihood of the conditioning parameters
%    where the m-th indicator process I is integrated out
% 2. sp might give a wrong value if you canceled constants when computing the likelihood
%    (note that even 1/sqrt(2*pi) matters in a setting with a random partion of the data
%    depending on some of the parameters of interest)

% Author: Sylvia Fruehwirth-Schnatter
% Last change: 23. September 2001

T=size(lik,3);
M=size(lik,2);   % number of indicators
L=size(lik,1); % total number of states

  maxl = max(lik,[],1);
   p =  exp(lik-maxl(ones(L,1),:,:));
   % p is the non-normalized posterior of I(t,:) 
   % (we did a numerical trick to avoid underflow)
   cp=sum(p,1);
    p = p./cp(ones(L,1),:,:);  

    Imc = squeeze(sum(cumsum(p) < rand(L,M,T))) + 1; 
    if M==1 Imc=Imc';end
pd=log(cp)+maxl;