function [I]= simstate_ms_prior(M,xi,T)

% simulate multivariate (M) indicators with L states 
% from the Markov switching prior defined by xi

% (all indicators are assumed to start in state 1 with probability 1)


% Sampling method used:
% 1. multi move sampling (forward filtering - backward sampling)
% 2. if xi is the same for all processes the procedure is vectorized and 
%    samples all components I(t,:) simultaniously, 
%    otherwise there is an additional loop over m=1,...,M

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input: 

% M .. number of processes

% xi .. transition matrix  of  a processes with L states

% Comments:
% 1. If xi is an L x L array  (meaning that size(xi,3)=1),
%    then xi is assumed to be the same for all processes.

%    row k of xi corresponds to the conditional probability of I(t,m) given I(t-1,m)=k
%    for arbitrary m


% 2. otherwise xi is a L x L x M with M=size(xi,3)>1,
%    then xi is assumed to be different for the various processes 

%    row k of xi(:,:,m) corresponds to the conditional probability of I(t,m) given I(t-1,m)=k

% T ... lenght of the simulated path
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Output:

% I ... path of the indicator sampled from the prior (T times M);
% Comments:
% 1.  I(t,m) refers to the value of process m at time t

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Sylvia Fruehwirth-Schnatter
% Last change: 23. September 2001

L=size(xi,1); % total number of states


p_pred=ones(L,M)/L;
 I(1,:) = sum(cumsum(p_pred) < rand(L,M)) + 1;    


for t=2:T+1
    for m=1:M
        
        if size(xi,3)==1  
            p_pred(:,m) = xi(I(t-1,m),:)';   % p_pred is the predicitve density
        elseif size(xi,3)==M
            p_pred(:,m) = squeeze(xi(I(t-1,m),:,m))';  
    else
        'xi and M do not agree in simstae_ms_prior'
    end  
            end     

    I(t,:) = sum(cumsum(p_pred) < rand(L,M)) + 1;    
    
    
end   

I=I(2:end,:);
